'use strict'

/**
 * 软件安装检测
 *
 * @author EllisRan.
 * @date 2018/8/9
 * @version
 */
import db from '../../database'
const path = require('path')
const ffi = require('ffi')
const fs = require('fs')
const electron = require('electron')
const ipcMain = electron.ipcMain
const regeditPath64 = 'HKLM\\SOFTWARE\\WOW6432Node\\Microsoft\\Windows\\CurrentVersion\\Uninstall'
const regeditPath32 = 'HKLM\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall'
const regeditPath = isOSWin64() ? regeditPath64 : regeditPath32
const regedit = require('regedit')

const exeinstall = function (win) {
  ipcMain.on('checkExeInstall', (event, args) => {
    let info = args.info
    try {
      // 先检查数据库中是否已存在该注册表软件的信息，有则退出，无则继续
      let exesqlinfo = db.read().get('sc_exeData').find({ regeditName: info.regedit, product_version: info.version }).value()
      if (exesqlinfo !== undefined) {
        event.sender.send('InstallResult', { status: 400, msg: `启动器已存在相同版本的 ${info.fulltitle} 软件`, id: info.id, index: args.index })
      } else {
        let regeditpath = path.join(regeditPath, info.regedit)
        regedit.list(regeditpath, async function (err, result) {
          if (err === null && result !== undefined) {
            let values = result[regeditpath].values
            let regeditMsg = {
              path: values['Inno Setup: App Path'].value,
              name: values['DisplayName'].value,
              regeditName: info.regedit,
              fileName: values['Inno Setup: Icon Group'].value,
              simpleName: info.product_title,
              versionName: info.title,
              keyNumber: info.key_number,
              keytype: info.keytype,
              isshow: true,
              auto: true,
              pid: info.pid,
              product_id: info.product_id,
              down_id: info.down_id,
              show_updateVersion: info.version
            }
            let data = await checkDirectory(regeditMsg.path)
            for (let i = 0; i < data.length; i++) {
              let exeinfo = regeditMsg
              exeinfo.exeName = data[i].exeName
              exeinfo.fileVersion = data[i].FileVersion
              exeinfo.productName = data[i].ProductName
              exeinfo.productVersion = data[i].ProductVersion
              exeinfo.fileDescription = data[i].FileDescription
              exeinfo.addtime = Date.parse(new Date()) / 1000
              exeinfo.product_version = data[i].FileVersion
              exeinfo.show_tip = data[i].FileVersion !== info.version && data[i].exeName !== 'BillsEditor.exe'
              exeinfo.id = GetFileNameNoExt(exeinfo.exeName) + '-' + RndNum(10)
              db.read().get('sc_exeData').insert(exeinfo).write()
            }
            // 查找是否存在旧版本的软件，升级后判断旧版本(相同文件夹升级则删除，不同文件夹则保留)
            let exedatalist = await db.read().get('sc_exeData').filter({ product_id: info.product_id, down_id: info.down_id }).value()
            for (let j in exedatalist) {
              if (exedatalist[j].product_version !== info.version && exedatalist[j].path === regeditMsg.path) {
                await db.read().get('sc_exeData').updateById(exedatalist[j].id, { isshow: false }).write()
              } else {
                // 把旧版本的更新提示设为false
                await db.read().get('sc_exeData').updateById(exedatalist[j].id, { show_tip: false }).write()
              }
            }
            await db.read().get('sc_exeData').removeWhere({ isshow: false }).write()

            await db.read().get('sc_productData').updateById(info.pid, { isshow: true }).write()
            await db.read().set('sc_hadInstall.first', false).write()
            await db.read().set('sc_hadInstall.url', '/softwarestartup/' + info.pid).write()
            event.sender.send('InstallResult', { status: 200, msg: data.length, id: info.id, index: args.index })
            win.webContents.send('successUpdate', { id: info.pid, num: data.length, delnum: 0 })
          } else {
            event.sender.send('InstallResult', { status: 400, msg: `未能成功安装 ${info.fulltitle} 软件`, id: info.id, index: args.index })
          }
        })
      }
    } catch (err) {
      event.sender.send('InstallResult', { status: 400, msg: err, id: info.id, index: args.index })
    }
  })

  // 取文件名不带后缀
  function GetFileNameNoExt (filepath) {
    if (filepath !== '') {
      let names = filepath.split('\\')
      let pos = names[names.length - 1].lastIndexOf('.')
      return names[names.length - 1].substring(0, pos)
    }
  }

  // 产生随机数函数
  function RndNum (n) {
    let rnd = ''
    for (let i = 0; i < n; i++) {
      rnd += Math.floor(Math.random() * 10)
    }
    return rnd
  }

  /**
   * 判断文件名是否包含纵横软件exe的部分名称
   * @param item
   * @return {boolean}
   */
  function existSoftwareName (item) {
    // 纵横软件exe所带包含的文件名
    const smartcostSoftwareHeader = ['SmartCost', 'Measure', 'DrawingBuilder', 'BillsEditor']
    let flag = false
    for (let i = 0; i < smartcostSoftwareHeader.length; i++) {
      if (item.indexOf(smartcostSoftwareHeader[i]) !== -1) {
        flag = true
        break
      }
    }
    return flag
  }

  /**
   * 检查文件夹并判断是否包含纵横软件(一个文件可能存在多个纵横软件的)
   * @param files
   * @return {Promise.<Array>}
   */
  async function checkDirectory (files) {
    let data = []
    let dirname = await fs.readdirSync(files)
    dirname.forEach(async function (item, index) {
      if (path.extname(item) === '.exe' && existSoftwareName(item)) {
        let pathstr = path.join('data/fileInfo.dll')
        let libm = ffi.Library(pathstr, {
          'GetFileInfo': ['string', ['string']]
        })
        let fileinfo = JSON.parse(await libm.GetFileInfo(path.join(files, item)))
        fileinfo.exeName = item
        data.push(fileinfo)
      }
    })
    return data
  }
}

/**
 * 判断系统位数
 * @return {boolean}
 */
function isOSWin64 () {
  return process.arch === 'x64' || process.env.hasOwnProperty('PROCESSOR_ARCHITEW6432')
}

export default exeinstall
