'use strict'

/**
 * 手动添加纵横软件文件夹
 *
 * @author EllisRan.
 * @date 2018/6/20
 * @version
 */
import db from '../../database'
const path = require('path')
const ffi = require('ffi')
const fs = require('fs')
const fse = require('fs-extra')
const electron = require('electron')
const ipcMain = electron.ipcMain
const dialog = electron.dialog
const regeditPath64 = 'HKLM\\SOFTWARE\\WOW6432Node\\Microsoft\\Windows\\CurrentVersion\\Uninstall'
const regeditPath32 = 'HKLM\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall'
const regeditPath = isOSWin64() ? regeditPath64 : regeditPath32
const regedit = require('regedit')

const fileselect = function (win) {
  ipcMain.on('file-select', function (event) {
    dialog.showOpenDialog({
      properties: ['openFile', 'openDirectory']
    }, async function (files) {
      if (files) {
        let data = await checkDirectory(files[0])
        if (data.length === 0) {
          event.sender.send('failedDirectory', { msg: '该文件夹不存在纵横软件' })
        } else {
          let addnum = await checkExeName(files[0], data)
          if (addnum !== 0) {
            let info = await db.read().get('sc_exeData').last().value()
            await db.read().get('sc_productData').updateById(info.pid, { isshow: true }).write()
            await db.read().set('sc_hadInstall.url', '/softwarestartup/' + info.pid).write()
            event.sender.send('successUpdate', { id: info.pid, num: addnum, delnum: 0 })
          } else {
            event.sender.send('failedDirectory', { msg: '该文件夹中的纵横软件已存在启动器中' })
          }
        }
      } else {
        event.sender.send('failedDirectory', { msg: '' })
      }
    })
  })
}

/**
 * 检查文件夹并判断是否包含纵横软件(一个文件可能存在2个纵横软件的)
 * @param files
 * @return {Promise.<Array>}
 */
async function checkDirectory (files) {
  let data = []
  let dirname = await fs.readdirSync(files)
  dirname.forEach(async function (item, index) {
    if (path.extname(item) === '.exe' && existSoftwareName(item)) {
      let pathstr = path.join('data/fileInfo.dll')
      let libm = ffi.Library(pathstr, {
        'GetFileInfo': ['string', ['string']]
      })
      let fileinfo = JSON.parse(await libm.GetFileInfo(path.join(files, item)))
      fileinfo.exeName = item
      data.push(fileinfo)
    }
  })
  return data
}

/**
 * 判断数据库是否已存在该文件信息,不存在则添加到数据库中
 * @param Directory
 * @param files
 * @return {Promise.<number>}
 */
async function checkExeName (Directory, files) {
  let index = Directory.lastIndexOf('\\')
  let filename = Directory.substring(index + 1, Directory.length)
  let addnum = 0
  let addflag = false
  for (let i = 0; i < files.length; i++) {
    let exeinfo = await db.read().get('sc_exeData').find({ path: Directory, exeName: files[i].exeName, fileVersion: files[i].FileVersion }).value()
    if (!exeinfo) {
      addflag = true
      break
    }
  }
  if (addflag) {
    // 再和注册表内容对应，有则添加，无则为未分类
    let regeditMsg = await checkRegeditbyDirectory(Directory)
    await sleep(5000)
    for (let i = 0; i < files.length; i++) {
      let exeinfo = await db.read().get('sc_exeData').find({ path: Directory, exeName: files[i].exeName, fileVersion: files[i].FileVersion }).value()
      if (!exeinfo) {
        let exeData = {
          path: Directory,
          name: regeditMsg.name !== '' ? regeditMsg.name : filename,
          regeditName: regeditMsg.regeditName,
          fileName: regeditMsg.fileName !== '' ? regeditMsg.fileName : filename,
          simpleName: regeditMsg.simpleName,
          versionName: regeditMsg.versionName,
          exeName: files[i].exeName,
          fileVersion: files[i].FileVersion,
          productName: files[i].ProductName,
          productVersion: files[i].ProductVersion,
          fileDescription: files[i].FileDescription,
          pid: regeditMsg.pid,
          keyNumber: regeditMsg.keyNumber,
          keytype: regeditMsg.keytype,
          product_id: regeditMsg.product_id,
          down_id: regeditMsg.down_id,
          product_version: files[i].FileVersion,
          show_tip: regeditMsg.show_updateVersion !== '' && files[i].FileVersion !== regeditMsg.show_updateVersion && files[i].exeName !== 'BillsEditor.exe',
          show_updateVersion: regeditMsg.show_updateVersion,
          addtime: Date.parse(new Date()) / 1000,
          isshow: true,
          auto: regeditMsg.auto,
          id: GetFileNameNoExt(files[i].exeName) + '-' + RndNum(10)
        }
        await db.read().get('sc_exeData').insert(exeData).write()
        ++addnum
      }
    }
  }
  return addnum
}

const sleep = (timeout = 2000) => new Promise(resolve => {
  setTimeout(resolve, timeout)
})

// 取文件名不带后缀
function GetFileNameNoExt (filepath) {
  if (filepath !== '') {
    let names = filepath.split('\\')
    let pos = names[names.length - 1].lastIndexOf('.')
    return names[names.length - 1].substring(0, pos)
  }
}

// 产生随机数函数
function RndNum (n) {
  let rnd = ''
  for (let i = 0; i < n; i++) {
    rnd += Math.floor(Math.random() * 10)
  }
  return rnd
}

/**
 * 判断文件名是否包含纵横软件exe的部分名称
 * @param item
 * @return {boolean}
 */
function existSoftwareName (item) {
  // 纵横软件exe所带包含的文件名
  const smartcostSoftwareHeader = ['SmartCost', 'Measure', 'DrawingBuilder', 'BillsEditor']
  let flag = false
  for (let i = 0; i < smartcostSoftwareHeader.length; i++) {
    if (item.indexOf(smartcostSoftwareHeader[i]) !== -1) {
      flag = true
      break
    }
  }
  return flag
}

function inArray (val, arr) {
  for (let i in arr) {
    if (arr[i] === val) {
      return true
    }
  }
  return false
}

/**
 * 纵横注册表信息中是否存在该文件夹，存在则返回该注册表信息,并加入到产品数据库中
 * @param directory
 * @return {{name: string, regeditName: string, simpleName: string, versionName: string, pid: string, keyNumber: string, keytype: string}}
 */
async function checkRegeditbyDirectory (directory) {
  let regeditMsg = {
    name: '',
    fileName: '',
    regeditName: '',
    simpleName: '',
    versionName: '',
    pid: '1',
    keyNumber: '',
    keytype: '',
    product_id: '',
    down_id: '',
    show_updateVersion: '',
    auto: false
  }
  regedit.list(regeditPath).on('data', async function (result) {
    try {
      let softwarelist = result.data.keys
      for (let i in softwarelist) {
        if (softwarelist[i].indexOf('SmartCost_') !== -1) {
          regedit.list(regeditPath + '\\' + softwarelist[i]).on('data', async function (result2) {
            let values = result2.data.values
            let directoryPath = values['Inno Setup: App Path'].value
            if (directoryPath === directory) {
              regeditMsg.name = values['DisplayName'].value
              regeditMsg.fileName = values['Inno Setup: Icon Group'].value
              regeditMsg.regeditName = softwarelist[i]
              let softwarejson = path.join('data/sc_software.json')
              let downlist = fse.readJsonSync(softwarejson).sc_down
              let downinfo = downlist.find(function (item) {
                return inArray(regeditMsg.regeditName, item.regedits)
              })
              // let ptitle = downinfo !== undefined ? downinfo.product_title : ''
              // 查找数据库是否存在该产品，有则添加，无则根据simplename 新增产品，simplename为空的话，则为未分类
              // 新方法，先关联注册表名查找数据，然后再关联产品数据库生成
              if (downinfo !== undefined) {
                regeditMsg.simpleName = downinfo.product_title
                regeditMsg.versionName = downinfo.title
                regeditMsg.auto = true
                let productInfo = db.read().get('sc_productData').find({ product_id: downinfo.product_id }).value()
                if (productInfo !== undefined) {
                  regeditMsg.pid = productInfo.id
                  db.read().get('sc_productData').updateById(productInfo.id, { isshow: true }).write()
                } else {
                  // 查找sc_software.json文件，有则新增产品，无则为未分类（旧的也会归这类）
                  let scproductlist = fse.readJsonSync(softwarejson).sc_product
                  let scproductinfo = scproductlist.find(function (item) {
                    return item.product_id === downinfo.product_id
                  })
                  if (scproductinfo !== undefined) {
                    let addproduct = scproductinfo
                    addproduct.addtime = Date.parse(new Date()) / 1000
                    addproduct.isshow = false
                    let info = db.read().get('sc_productData').insert(addproduct).write()
                    regeditMsg.pid = info.id
                  }
                }
                // 锁号和锁类型添加
                // let downlist = fse.readJsonSync(softwarejson).sc_down
                // let downinfo = downlist.find(function (item) {
                //   return item.product_title === regeditMsg.simpleName && item.title === regeditMsg.versionName
                // })
                regeditMsg.keyNumber = downinfo.key_number
                regeditMsg.keytype = downinfo.keytype
                regeditMsg.product_id = downinfo.product_id
                regeditMsg.down_id = downinfo.down_id
                regeditMsg.show_updateVersion = downinfo.version
              }
              return regeditMsg
            }
          })
        }
      }
    } catch (err) {
      console.log(err)
    }
  })
  return regeditMsg
}

/**
 * 判断系统位数
 * @return {boolean}
 */
function isOSWin64 () {
  return process.arch === 'x64' || process.env.hasOwnProperty('PROCESSOR_ARCHITEW6432')
}

export default fileselect
